﻿using MediatR;
using Microsoft.EntityFrameworkCore;
using Mirea.Api.DataAccess.Application.Common.Exceptions;
using Mirea.Api.DataAccess.Application.Interfaces.DbContexts.Schedule;
using System.Threading;
using System.Threading.Tasks;

namespace Mirea.Api.DataAccess.Application.Cqrs.TypeOfOccupation.Commands.CreateTypeOfOccupation;

public class CreateTypeOfOccupationCommandHandler(ITypeOfOccupationDbContext dbContext) : IRequestHandler<CreateTypeOfOccupationCommand, int>
{
    public async Task<int> Handle(CreateTypeOfOccupationCommand request, CancellationToken cancellationToken)
    {
        var entity = await dbContext.TypeOfOccupations.FirstOrDefaultAsync(t => t.ShortName == request.ShortName, cancellationToken: cancellationToken);

        if (entity != null)
            throw new RecordExistException(typeof(Domain.Schedule.TypeOfOccupation), nameof(Domain.Schedule.TypeOfOccupation.ShortName), entity.Id);

        var typeOfOccupation = new Domain.Schedule.TypeOfOccupation
        {
            ShortName = request.ShortName,
            FullName = request.FullName
        };

        var result = await dbContext.TypeOfOccupations.AddAsync(typeOfOccupation, cancellationToken);
        await dbContext.SaveChangesAsync(cancellationToken);

        return result.Entity.Id;
    }
}